from .assistant import AssistantCommand, AssistantManager, TranscriptEntry
from .clipboard import ClipboardHelper
from .config import ConfigManager, WorkspaceProfile, WorkspaceProfileManager
from .core import App, ShortcutManager, Tab, Window
from .events import EventBus
from .core import App
from .integrations import ProcessTracker, ProjectInfo, detect_project_type, detect_status, find_startup_target, project_rows, scan_projects
from .ops import BackgroundJobManager, DependencyGraph, HealthSuite, PortChecker, ProcessMonitor, RecentProjects, Scheduler, ServiceController, Task, TaskResult, TaskRunner
from .plugins import CustomWidgetSpec, Plugin, PluginManager, PluginMetadata, PluginSandbox, WidgetRegistry
from .theme import Theme, ThemeEditor, get_theme
from .validators import SchemaValidator, choice_field, email_field, float_field, int_field, min_length, path_field, required, url_field, validator_for
from .voice import LANGUAGE_PHRASE_PACKS, VoiceEngine, VoiceRouter, VoiceRule, find_best_phrase, normalize_spoken_text, similarity_score
from .widgets import (
    AssistantPanel,
    Badge,
    Button,
    Clock,
    CommandPalette,
    ContextAction,
    ContextMenu,
    CustomWidget,
    Divider,
    FilePicker,
    Form,
    Input,
    JSONViewer,
    KeyValue,
    ListWidget,
    LogViewer,
    Logger,
    MultiSelect,
    Paragraph,
    Progress,
    Section,
    Select,
    SettingPage,
    StatCard,
    StatusTable,
    SubTab,
    SubTabView,
    Table,
    TerminalOutputPanel,
    Text,
    Toggle,
    TranscriptPanel,
    TreeNode,
    TreeWidget,
    VoiceCommand,
)

__all__ = [
    'AssistantManager', 'AssistantCommand', 'TranscriptEntry',
    'App', 'Window', 'Tab', 'ShortcutManager', 'Theme', 'ThemeEditor', 'get_theme', 'EventBus',
    'ClipboardHelper', 'ConfigManager', 'WorkspaceProfile', 'WorkspaceProfileManager',
    'Task', 'TaskResult', 'TaskRunner', 'BackgroundJobManager', 'ServiceController', 'ProcessMonitor', 'PortChecker', 'HealthSuite', 'DependencyGraph', 'Scheduler', 'RecentProjects',
    'Plugin', 'PluginMetadata', 'PluginManager', 'PluginSandbox', 'CustomWidgetSpec', 'WidgetRegistry',
    'Text', 'Paragraph', 'Divider', 'Button', 'Toggle', 'Input', 'Select', 'MultiSelect', 'Badge', 'Progress', 'Table', 'StatusTable',
    'Logger', 'LogViewer', 'TerminalOutputPanel', 'CommandPalette', 'ListWidget', 'TreeNode', 'TreeWidget', 'Clock', 'Form', 'SettingPage',
    'ContextAction', 'ContextMenu', 'KeyValue', 'StatCard', 'FilePicker', 'JSONViewer', 'SubTab', 'SubTabView', 'CustomWidget', 'VoiceCommand', 'VoiceRule', 'AssistantPanel', 'TranscriptPanel',
    'ProcessTracker', 'ProjectInfo', 'detect_project_type', 'detect_status', 'find_startup_target', 'project_rows', 'scan_projects',
    'VoiceEngine', 'VoiceRouter', 'LANGUAGE_PHRASE_PACKS', 'normalize_spoken_text', 'similarity_score', 'find_best_phrase',
    'SchemaValidator', 'required', 'min_length', 'int_field', 'float_field', 'email_field', 'url_field', 'path_field', 'choice_field', 'validator_for',
]
